#include <string.h>

#include <libgnomevfs/gnome-vfs.h>
#include <gnome.h>
#include <panel-applet.h>
#include <gtk/gtklabel.h>

#include "wattlet.h"

gboolean parse_battery(WattletApplet *applet, int *mwatts)
{
	GnomeVFSHandle   *handle;
	GnomeVFSFileSize  bytes_read;
	GnomeVFSResult    result;
	gchar            *uri, *file;
	gchar            **lines;
	gchar             buffer[256];
	gint              i;
	gint              sign=1;

	g_return_val_if_fail (PANEL_IS_APPLET (PANEL_APPLET (applet)), FALSE);

	uri = gnome_vfs_get_uri_from_local_path ("/proc/acpi/battery/BAT0/state");

	result = gnome_vfs_open (&handle, uri, GNOME_VFS_OPEN_READ);
	if (result != GNOME_VFS_OK) {
		if (uri) g_free (uri);

		return FALSE;
	}

	g_free (uri);

	result = gnome_vfs_read (handle, buffer, 256, &bytes_read);
	file = g_strndup (buffer, bytes_read);
	if (result != GNOME_VFS_OK) {
		g_free (file);

		return FALSE;
	}

	result = gnome_vfs_close (handle);
	if (result != GNOME_VFS_OK) {
		g_free (file);

		return FALSE;
	}

	lines = g_strsplit (file, "\n", -1);
	for (i=0; lines[i]; i++) {
		if (g_ascii_strncasecmp (lines[i], "present rate", 12) == 0) 
		{
			sscanf (lines[i], "present rate: %d mW",
				mwatts);
		}
		if (g_ascii_strncasecmp (lines[i], "charging state", 14) == 0) 
		{
			sscanf (lines[i], "charging state: %s",
				buffer);
			if (g_ascii_strncasecmp (buffer, "discharging", 11) == 0)
				sign=-1;
			else
				sign=1;
		}
	}

	*mwatts *= sign;
	g_strfreev (lines);
	g_free (file);

	return TRUE;
}

gboolean wattlet_update(gpointer gdata)
{
	WattletApplet *applet=(WattletApplet*) gdata;
	int mwatts = 0;
	
	parse_battery(applet, &mwatts);

	gtk_label_set_text (GTK_LABEL (applet->label), g_strdup_printf("%.2f W", mwatts/1000.0f));

	return TRUE;
}


static gboolean
wattlet_applet_fill (WattletApplet *applet,
		     const gchar *iid,
		     gpointer data)
{
	printf("1\n");
	
	if (strcmp (iid, "OAFIID:WattletApplet") != 0)
		return FALSE;
	
	applet->label = gtk_label_new ("??.?? W");
	printf("2\n");
	
	gtk_container_add (GTK_CONTAINER (applet), applet->label);
	
	gtk_widget_show_all (GTK_WIDGET (applet));
	
	applet->timeout_handler = g_timeout_add(1500, wattlet_update, (gpointer) applet);

	return TRUE;
}

static GType wattlet_applet_get_type (void)
{
	static GType type = 0;

	if (!type) {
		static const GTypeInfo info = {
			sizeof (PanelAppletClass),
			NULL, NULL, NULL, NULL, NULL,
			sizeof (WattletApplet),
			0, NULL, NULL
		};

		type = g_type_register_static (
			PANEL_TYPE_APPLET, "WattletApplet", &info, 0);
	}

	return type;
}

PANEL_APPLET_BONOBO_FACTORY ("OAFIID:WattletApplet_Factory",
			     wattlet_applet_get_type(),
			     //                             PANEL_TYPE_APPLET,
                             "Wattlet",
                             "0",
                             wattlet_applet_fill,
                             NULL);
